require 'cucumber-rest-bdd'

Given("I am an API management authenticated client") do
    if ENV['apim_subscription_key'] != nil && ENV['apim_subscription_key'] != ""
      @apim_subscription_key = ENV['apim_subscription_key']
    else
      variables = %w{ALINTA_TENANTID ALINTA_CLIENTID ALINTA_CLIENTSECRET ALINTA_VAULT}
      missing = variables.find_all { |v| ENV[v] == nil }
      unless missing.empty?
        raise "The following required environment variables have not been defined: #{missing.join(', ')}."
      end
      steps %Q{
        Given I retrieve the API Management subscription key secret "APIMgmnt--End2End--Unlimited--Primary" from Azure Storage Vault "#{ENV['ALINTA_VAULT']}" using tenant "#{ENV['ALINTA_TENANTID']}" with credentials "#{ENV['ALINTA_CLIENTID']}" and "#{ENV['ALINTA_CLIENTSECRET']}"
      }
    end
    steps %Q{
      Given I am a client
      And I add the API Management key header
    }
  end
  
Given("I retrieve the API Management subscription key secret {string} from Azure Storage Vault {string} using tenant {string} with credentials {string} and {string}") do |secret_name, vault_name, tenant_id, client_id, client_secret|
    if @apim_subscription_key.to_s.empty?
        steps %Q{
            Given I retrieve the secret "#{secret_name}" from Azure Storage Vault "#{vault_name}" using tenant "#{tenant_id}" with credentials "#{client_id}" and "#{client_secret}"
        }
        @apim_subscription_key = @response.get_as_type "$..value", "string"
    end
end

Given("I add the API Management key header") do
    if ENV['api_synthetic_requests'] != nil && ENV['api_synthetic_requests'] == "true"
        steps %Q{
            I enable synthetic requests
        }
    end
    steps %Q{
        And I add Headers:
        | Ocp-Apim-Subscription-Key | #{@apim_subscription_key} |
    }
end

Given("I enable synthetic requests") do
    steps %Q{
        And I add Headers:
        | is-synthetic              | true |
    }
end

Given("I retrieve the secret {string} from Azure Storage Vault {string} using tenant {string} with credentials {string} and {string}") do |secret_name, vault_name, tenant_id, client_id, client_secret|
    steps %Q{
        Given I authenticate with Azure tenant "#{tenant_id}" using client credentials "#{client_id}" and "#{client_secret}"
    }
    access_token = @response.get_as_type "$..access_token", "string"
    steps %Q{
        Given I retrieve the secret "#{secret_name}" from Azure Storage Vault "#{vault_name}" using access token "#{access_token}"
    }
end

Given("I authenticate with Azure tenant {string} using client credentials {string} and {string}") do |tenant_id, client_id, client_secret|
    steps %Q{
        Given I authenticate with "https://login.windows.net/#{tenant_id}/oauth2/token" using client credentials "#{client_id}" and "#{client_secret}"
    }
end

Given("I authenticate with {string} using client credentials {string} and {string}") do |url, client_id, client_secret|
    steps %Q{
        Given I send "www-x-form-urlencoded" and accept JSON
        When I set form request body to:
        | grant_type    | client_credentials      |
        | client_id     | #{client_id}            |
        | client_secret | #{client_secret}        |
        | resource      | https://vault.azure.net |
        And I send a POST request to "#{url}"
        Then the request was successful
    }
end

Given("I retrieve the secret {string} from Azure Storage Vault {string} using access token {string}") do |secret_name, vault_name, access_token|
    api_version = '2015-06-01'
    url = "https://#{vault_name}.vault.azure.net/secrets/#{secret_name}?api-version=#{api_version}"
    steps %Q{
        Given I send and accept JSON
        And I add Headers:
        | Authorization | Bearer #{access_token} |
        When I send a GET request to "#{url}"
        Then the request was successful
    }
end

Given("I am issuing versioned requests") do
end

Given("I am issuing versioned requests for {resource_name}") do |resource|
    @urlbasepath = resource
    @urlbasepath = "#{@urlbasepath}/" unless @urlbasepath.end_with?('/')
end

